package Net::Statsd::Tiny::Types;

# ABSTRACT: This is no longer used

use strict;
use warnings;

our $VERSION = 'v0.3.1';


1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Net::Statsd::Tiny::Types - This is no longer used

=head1 VERSION

version v0.3.1

=head1 DESCRIPTION

This module has been renamed to L<Net::Statsd::Lite::Types>.

=head1 SOURCE

The development version is on github at L<https://github.com/robrwo/Net-Statsd-Tiny>
and may be cloned from L<git://github.com/robrwo/Net-Statsd-Tiny.git>

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website
L<https://github.com/robrwo/Net-Statsd-Tiny/issues>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

Robert Rothenberg <rrwo@cpan.org>

The initial development of this module was sponsored by Science Photo
Library L<https://www.sciencephoto.com>.

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2018 by Robert Rothenberg.

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut
